@@ -22,14 +22,7 @@
 //
 // You should have received a copy of the GNU General Public License
 //--------------------------------------------------------------------------
-#[macro_use]
-extern crate static_math;
-
-use static_math::matrix2x2::M22;
-use static_math::matrix3x3::M33;
-use static_math::matrix4x4::M44;
-use static_math::matrix5x5::M55;
-use static_math::matrix6x6::M66;
+use static_math::{M22, M33, M44, M55, M66, m22_new, m33_new, m44_new, m55_new, m66_new};
 use static_math::traits::LinearAlgebra;
 
 fn main() {
